;+-----------------------------------------------------------------------------
; isr9.asm
;+-----------------------------------------------------------------------------
    ideal
    model     small
    P286N

dataseg

    _ISR9_intr_count    dw   00h            ; Counter of keyboard interrupts

codeseg

    _Original_09h       dd   ?              ; Put original Keyboard handler
                                            ;  here.
;   --  Publics

    public              _kcount, _install_ISR9, _remove_ISR9

    extrn               _signal_reader:proc
    
;+-----------------------------------------------------------------------------
; ISR9   --   Invokes the orignal keyboard handler then calls a C-language
;             routine, signal_reader() that wakes up the reader thread.
;+-----------------------------------------------------------------------------

proc     ISR9

;
;   --   dispatch the original interrupt routine
;
         pushf
         call [dword ptr cs:_Original_09h]

         inc  [cs:_ISR9_intr_count]

;
;   --   We're back from the ISR.  Save our state and then check to see 
;        whether a character is waiting in the keyboard input buffer
;

         pusha
         push ss
         push es
         push ds

         mov  ah, 1                    
         int  16h
         jz   @@done                        ; Return to interrupted routine
                                            ;   if no characters are pending
                                            ; Else
         call _signal_reader                ;   Chars are pending.  Signal the
                                            ;   the reader thread.
;
;   --   We're all done.  Restore the ctxsw flag to its state upon entry.
;

@@done:

         pop  ds
         pop  es
         pop  ss
         popa         

         iret
         
endp     ISR9

;+---------------------------------------------------------------------------
; _install_ISR9
;
; PROTOTYPE:     void install_ISR9( void )
;+---------------------------------------------------------------------------

proc     _install_ISR9

         push bp
         mov  bp, sp              ; set up a normal procedure stack

         push bx
         push es
         push dx
         push ds
         
;
;   --   Get the existing 09h timer interrupt vector (returned in registers 
;        ES:BX by interrupt function 35h) and copy it into memory.  We will
;        use this to restore 09h when we exit.

         mov  ax, 3509h
         int  21h

         mov  [word ptr cs:_Original_09h], bx
         mov  [word ptr cs:_Original_09h + 2], es

;   --   Move our own handler, ISR9, into registers DS:DX and 
;        call the install interrupt function 25h.

         push cs
         pop  ds                       ; Make DS equal CS

         mov  dx, offset ISR9
         
         mov  ax, 2509h
         int  21h


         pop  ds
         pop  dx
         pop  es
         pop  bx

         pop  bp
         ret

endp     _install_ISR9

;+---------------------------------------------------------------------------
; _remove_ISR9
;+---------------------------------------------------------------------------

proc     _remove_ISR9

         push bp
         mov  bp, sp                        ; Set up normal stack frame

         push ds
         push dx
         
         mov  ds,  [word ptr cs:_Original_09h+2]
         mov  dx,  [word ptr cs:_Original_09h]

         mov  ax, 2509h
         int  21h
         
         pop  dx
         pop  ds
         pop  bp

         ret
         
endp     _remove_ISR9

;+----------------------------------------------------------------------------
; _kcount          NumChars = kcount();
;+----------------------------------------------------------------------------

proc     _kcount

    pushf
    cli
    
    mov  ax, [cs:_ISR9_intr_count]
    
    popf
    ret
    
endp      _kcount

    end       ; end of module isr9.asm
