/****************************************************************
 *                                                              *
 *  (C)  The TriTechnologies Company, 1991                      *
 *  	      	   	     	       	    	      	   	*
 *       This software is furnished under a license and may     *
 *       be used and copied only in accordance with the terms   *
 *       of such license and with the inclusion of the above    *
 *       copyright notice.  this software or any other copies   *
 *       thereof may not be provided or otherwise made avail-   *
 *       able to any other person.  No title to and ownership   *
 *       of the software is hereby transferred.                 *
 *                                                              *
 *       The information in this software is subject to change  *
 *       without notice and should not be construed as a        *
 *       commitment by The TriTechnologies Company.             *
 ****************************************************************
 *
 * Facility:
 *
 *     rwmain.c
 *
 * Abstract:
 *
 *       rwmain.c is the main program that illustrates the use
 *       of shared and exclusive locks.
 */

#include <stdlib.h>
#include <stdio.h>
#include "pcthread.h"

extern void lock_shared( void );
extern void unlock_shared( void );
extern void lock_exclusive( void );
extern void unlock_exclusive( void );
extern int create_rwlocks( void );

static pcthread_t       **rthread, **wthread, *timer_thread;

void reader( void );
void writer( void );
void timer (void );

static unsigned array[100];
static boolean_t done = false;

main( int argc, char *argv[] )
{
    pcthread_attr_t  *th_attr_handle;
    int status = 0, i, max_p, dummy, reader_threads = 5, writer_threads = 1;

    if( argc != 3 )
    {
         printf("rw [reader_threads] [writer_threads]\n");
         exit( 0 );
    }
    
    reader_threads = atoi( argv[1] );
    writer_threads = atoi( argv[2] );
    
    /*
     * Allocate memory for the pointers to the thread handles for the 
     * reader and writer threads.
     */
     
    rthread = (pcthread_t **) malloc( reader_threads * sizeof(pcthread_t *));
    for( i = 0; i < reader_threads; i++ )
         rthread[i] = (pcthread_t *) malloc( sizeof( pcthread_t *));
         
    wthread = (pcthread_t **) malloc( writer_threads * sizeof(pcthread_t *));
    for( i = 0; i < writer_threads; i++ )
         wthread[i] = (pcthread_t *) malloc( sizeof( pcthread_t *));

    /*
     * Initialize the global array
     */

    for(i = 0; i < 100; i++ )
         array[i] = i;

    /*
     * Set the priority of the current (i.e., the initial thread) such
     * that it may not be preempted by any of its children.
     */
     
    max_p = pcthread_attr_getmaxprio_ns();
    (void) pcthread_setactive_prio_ns( max_p );

    /*
     * Create the shared and exclusive locks
     */
     
    status = create_rwlocks();

    /*
     * Now, initialize the handle to the thread attributes object that 
     * we'll use in creating the reader, writer, and timer threads.  We'll
     * set up the attributes such as to create reader and writer threads
     * with relatively small stacks.
     */

    th_attr_handle = pcthread_attr_getdefault_ns();
    status = pcthread_attr_setstacksize( th_attr_handle, 256 );    

    for( i = 0; i < reader_threads; i++ )
         status = pcthread_create( rthread[i], 
                                  *th_attr_handle, 
                                  (pcthread_action_t) reader,
                                  (pcthread_addr_t *) &dummy);

    for(i = 0; i < writer_threads; i++ )
         status = pcthread_create( wthread[i], 
                                  *th_attr_handle, 
                                  (pcthread_action_t ) writer,
                                  (pcthread_addr_t *) &dummy);

    /*
     * We insure that the timer thread does not have to wait on the
     * ready queue when its timer expires.  To accomplish this we insure
     * that we create the timer thread at the maximum priority.  Also,
     * we give it the least amount of stack space we can get away with.
     */
     
    status = pcthread_attr_setprio( th_attr_handle, max_p );
    status = pcthread_attr_setstacksize( th_attr_handle, 128 );

    status = pcthread_create( timer_thread, 
                             *th_attr_handle,
                             (pcthread_action_t) timer, 
                             (pcthread_addr_t *) &dummy);

    /*
     * In order to insure that the initial thread is the very last to exit, we
     * join with the timer thread.  We need to be the last thread because
     * we have a big enough stack to accomodate the statistics and the
     * trace file when the system exits.
     */
     
    status = pcthread_join( timer_thread, (pcthread_addr_t *) &dummy );

    return 0;
}    

void
reader()
{
    int i;
    unsigned sum = 0;
    
    while( done != true )
    {
         lock_shared();

              for(i = 0; i < 100; i++ )
                   sum += i;

         unlock_shared();
    }
}

void
writer()
{
    int i;
    
    while( done != true )
    {
         pcthread_delay_ns( rand() % 200 );
         lock_exclusive();
         
              for(i = 0; i < 100; i++ )
                   array[i] = i;

         unlock_exclusive();
    }
}
    
void
timer()
{
    /*
     * Delay 5.0 seconds...
     */
     
    pcthread_delay_ns( 5000 );
    done = true;

    /*
     * We lower our priority and then yield in order to flush any other 
     * threads waiting on the ready queue.  Thus, all the reader and
     * writer threads will complete prior to the timer thread.  When
     * we complete, the initial thread, i.e., main(), will be allowed
     * to proceed.
     */
     
    (void) pcthread_setactive_prio_ns( 1 );
    pcthread_yield();
}