/*
 * Test11.c --  A test of priority context switching
 */
 
#include <stdio.h>
#include "pcthread.h"

#define THREADS 10

static pcthread_t       compute_th[THREADS];
static pcthread_attr_t  th_attr_handle;
static int              completed_threads = 0, dummy;
static boolean_t        done = false;

static pcthread_mutex_t lock;
static pcthread_cond_t  wait_till_done;


void
compute()
{
    int i, j;
    unsigned sum = 0;

    for( i = 0; i < 30000; i++ )
         for(j = 0; j < 100; j++ );
         {
              sum++;
              if( sum == 30000 )
                   sum = 0;
         }

    (void) pcthread_mutex_lock( (pcthread_mutex_t *) &lock );

         completed_threads++;

         if( completed_threads == THREADS )
         {
              done = true;
              (void) pcthread_cond_signal((pcthread_cond_t *)&wait_till_done);
         }

    (void) pcthread_mutex_unlock( (pcthread_mutex_t *) &lock );
}

main()
{
    int i, j, max_prio;
    unsigned sum = 0;

    /*
     * This first set of services cause the inital thread to execute at
     * the maximum allowable priority.  This prevents us from being
     * preempted by the children we're going to create a little bit later.
     */
     
    max_prio = pcthread_attr_getmaxprio_ns();
    (void) pcthread_setactive_prio_ns( (unsigned) max_prio );

    /*
     * Next, we create and initialize our mutex and condition variable.
     * We'll just use whatever default attributes the system provides.
     */
     
    (void) pcthread_mutex_init( (pcthread_mutex_t *) &lock,
                                pcthread_mutexattr_default );

    (void) pcthread_cond_init( (pcthread_cond_t *) &wait_till_done,
                               pcthread_condattr_default );

    /*
     * Now, we initialize the thread attributes handle we'll use to create
     * the children.  Note that we create the children with a priority
     * of 1 less than the initial thread.
     */     

    (void) pcthread_attr_create( (pcthread_attr_t *) &th_attr_handle );
    (void) pcthread_attr_setprio( (pcthread_attr_t *) &th_attr_handle, 
                                  max_prio - 1 );

    /*
     * Now, we create the child threads.  Each of them will execute the
     * "compute()" action routine defined above.
     */
     
    for(i = 0; i < THREADS; i++ )
         pcthread_create( (pcthread_t *) &compute_th[i],
                          th_attr_handle,
                          (pcthread_action_t) compute,
                          (pcthread_addr_t *) &dummy );

    /*
     * As the initial thread calculates this loop, the RTMS will decrement
     * its priority.  When its priority falls to below that of the children
     * it previously created, it will be preempted.
     */

    for( i = 0; i < 30000; i++ )
         for(j = 0; j < 200; j++ );
         {
              sum++;
              if( sum == 30000 )
                   sum = 0;
         }

    /*
     * Now, wait until the other 10 threads have completed
     */

    (void) pcthread_mutex_lock( (pcthread_mutex_t *) &lock );

         while( done == false )
              pcthread_cond_wait( (pcthread_cond_t *) &wait_till_done, 
                                  (pcthread_mutex_t *) &lock );

    (void) pcthread_mutex_unlock( (pcthread_mutex_t *) &lock );

    return 0;
}